#ifndef _KVI_IMAGELIBRARY_KDE_H_INCLUDED_
#define _KVI_IMAGELIBRARY_KDE_H_INCLUDED_

// =============================================================================
//
//      --- kvi_imagelibrary_kde.h ---
//
//   This file is part of the KVIrc IRC client distribution
//   Copyright (C) 2003 Robin Verduijn <robin@debian.org>
//
//   This program is FREE software. You can redistribute it and/or
//   modify it under the terms of the GNU General Public License
//   as published by the Free Software Foundation; either version 2
//   of the License, or (at your opinion) any later version.
//
//   This program is distributed in the HOPE that it will be USEFUL,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//   See the GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program. If not, write to the Free Software Foundation,
//   Inc, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//
// =============================================================================

#include <kpixmapsplitter.h>

/**
 * A class to manage multiple image libraries.<br>
 * The library is in fact a big image that contains<br>
 * several rows of other smaller images of the same size.<br>
 * The images inside the 'library' are indexed in the following way:<br>
 * <CODE><br>
 * -------------<br>
 * |...|...|...|<br>
 * |.0.|.1.|.2.|<br>
 * -------------<br>
 * |...|...|...|<br>
 * |.3.|.4.|.5.|<br>
 * -------------<br>
 * |...|...|...|<br>
 * </CODE><br>
 * The 'library' image should have a size that is a multiple
 * of the single image size. If not, the remaining part is ignored.
 * The first image always starts in the upper left corner.
 *
 * @short A class to manage multiple image libraries
 * @author Szymon Stefanek
 * @version 1.0
*/
class KviImageLibrary : public KPixmapSplitter
{
public:
	/**
	 * Creates an image library from a file on disk.
	 * You should at least check if the library was loaded by calling
	 * libraryLoaded(); If the library cannot be loaded, each call
	 * to getImage will return a 'white' pixmap with the requested size.
	 * The second path is tried if the first one is not ok.
	 */
	KviImageLibrary(const char *path1, const char *path2, int imageWidth, int imageHeight);

	/**
	 * Destroys the image library
	 */
	~KviImageLibrary();

public:
	/**
	 * Returns true if an image library has been loaded.
	 */
	bool libraryLoaded();

	/**
	 * Returns a pixmap image containing the library image at index zeroBasedIndex.<br>
	 * If the index is out of range, or there is no library image a blank 32x32 pixmap is generated.
	 */
	QPixmap getImage(int zeroBasedIndex);

protected:
	/**
	 * Loads a library by path. Returns true if the library can be loaded.<br>
	 * If the library cannot be loaded, the m_pLibrary field is set to 0.<br>
	 * Destroys any previous library loaded.
	 */
	bool loadLibrary(const char *path);

	/**
	 * Returns the number of available images in the library.
	 */
	int imageCount();
private:
	/**
	 * The number of images in the library
	 */
	int m_count;
};

#endif // _KVI_IMAGELIBRARY_KDE_H_INCLUDED_
